/*
 * Author: Jake G
 * Date: 2025
 * filename: ADC.c
 * description: module_purpose
 */

#ifndef __AVR_ATtiny13A__
#define __AVR_ATtiny13A__
#endif

#include "ADC.h"
#include "RegEdit.h"
#include "avr/io.h"

#define MAX_CHANNEL_NUM 3

static uint8_t GetChannelPinNum(uint8_t adc_chan) {
  switch (adc_chan) {
  case 0:
    return PB5;
  case 1:
    return PB2;
  case 2:
    return PB4;
  case 3:
    return PB3;
  default:
    return 255; /*return invalid pin num.*/
  }
}

static bool IsInvalidChannel(uint8_t adc_chan) {
  if (adc_chan > MAX_CHANNEL_NUM) {
    return true;
  }
  return false;
}

void ADC_Setup(void) {
  // Clear the register, set VCC as ref, ADC0 as channel and
  // leave result right adjusted.
  RegEdit_ClearRegister((void *)&ADMUX);

  // Status & Control register A
  // Turn on the 8Div prescaler
  RegEdit_SetNum((void *)&ADCSRA, (1 << ADPS1) | (1 << ADPS0));

  // Status & Control register B
  // Turn on auto-trigger using time compare/match B.
  RegEdit_AND_Num((void *)&ADCSRB, (1 << ADTS2) | (0 << ADTS1) | (1 << ADTS0));
}

void ADC_Init(uint8_t adc_chan) {

  if (IsInvalidChannel(adc_chan)) {
    return;
  }

  // set the direction to input
  RegEdit_ClearBit((void *)&DDRB, adc_chan);

  // Disable the pull-up resistor
  RegEdit_ClearBit((void *)&PORTB, adc_chan);

  // Disable input buffer
  // We do some kinda nasty address addition but it saves
  // memory and means we don't need a switch statment.
  // RegEdit_SetBit((void *)(&PORTA.PIN0CTRL) + adc_chan,
  //               PORT_ISC_INPUT_DISABLE_gc);
}

void ADC_Enable(void) {
  // Set the enable bit in the CTRLA register
  // RegEdit_SetBit((void *)&ADC0.CTRLA, 0);
}

void ADC_Disable() {
  // Clear the enable ADC flag
  // RegEdit_ClearBit((void *)&ADC0.CTRLA, 0);
}

void ADC_SetPin(uint8_t adc_chan) {
  if (IsInvalidChannel(adc_chan)) {
    return;
  }
  // RegEdit_ClearRegister((void *)&ADC0.MUXPOS);
  // RegEdit_SetNum((void *)&ADC0.MUXPOS, adc_chan);
}

uint16_t ADC_ReadValue_Impl(uint8_t adc_chan) {
  // RegEdit_SetNum((void *)&ADC0.COMMAND, ADC_STCONV_bm);

  /* Wait until ADC conversion done */

  /*
   while (!(ADC0.INTFLAGS & ADC_RESRDY_bm)) {
   ;
 }
 */

  /* Clear the interrupt flag by writing 1: */
  // ADC0.INTFLAGS = ADC_RESRDY_bm;

  // uint16_t adc_val = (uint16_t)ADC0.RES;
  uint16_t adc_val = 0;
  adc_val = adc_val >> 5;
  return adc_val;
}

// Set the default for the function pointer.
uint16_t (*ADC_ReadValue)(uint8_t adc_chan) = ADC_ReadValue_Impl;
